"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const melonjs_1 = require("@melonproject/melonjs");
const deploymentContracts_1 = require("../utils/deploymentContracts");
var SetupProgress;
(function (SetupProgress) {
    SetupProgress["BEGIN"] = "BEGIN";
    SetupProgress["ACCOUNTING"] = "ACCOUNTING";
    SetupProgress["FEE_MANAGER"] = "FEE_MANAGER";
    SetupProgress["PARTICIPATION"] = "PARTICIPATION";
    SetupProgress["POLICY_MANAGER"] = "POLICY_MANAGER";
    SetupProgress["SHARES"] = "SHARES";
    SetupProgress["TRADING"] = "TRADING";
    SetupProgress["VAULT"] = "VAULT";
    SetupProgress["COMPLETE"] = "COMPLETE";
})(SetupProgress || (SetupProgress = {}));
exports.address = hub => hub.contract.address;
exports.name = (hub, _, context) => hub.getName(context.block);
exports.manager = (hub, _, context) => hub.getManager(context.block);
exports.creator = (hub, _, context) => hub.getCreator(context.block);
exports.creationTime = (hub, _, context) => hub.getCreationTime(context.block);
exports.isShutDown = (hub, _, context) => hub.isShutDown(context.block);
exports.routes = (hub, _, context) => __awaiter(void 0, void 0, void 0, function* () { return [hub, yield hub.getRoutes(context.block)]; });
exports.progress = (hub, _, context) => __awaiter(void 0, void 0, void 0, function* () {
    const hubRoutes = yield hub.getRoutes();
    const hubVersion = hubRoutes.version && new melonjs_1.Version(context.environment, hubRoutes.version);
    if (hubVersion && (yield hubVersion.isInstance(hub.contract.address, context.block))) {
        return SetupProgress.COMPLETE;
    }
    const version = yield deploymentContracts_1.versionContract(context.environment);
    const manager = yield hub.getManager(context.block);
    const managerRoutes = yield version.getManagersToRoutes(manager, context.block);
    if (managerRoutes.vault) {
        return SetupProgress.VAULT;
    }
    if (managerRoutes.trading) {
        return SetupProgress.TRADING;
    }
    if (managerRoutes.shares) {
        return SetupProgress.SHARES;
    }
    if (managerRoutes.policyManager) {
        return SetupProgress.POLICY_MANAGER;
    }
    if (managerRoutes.participation) {
        return SetupProgress.PARTICIPATION;
    }
    if (managerRoutes.feeManager) {
        return SetupProgress.FEE_MANAGER;
    }
    if (managerRoutes.accounting) {
        return SetupProgress.ACCOUNTING;
    }
    return SetupProgress.BEGIN;
});
